include("scripts/enums/JFActorDirection.lua")
include("scripts/enums/JMJobType.lua")
include("scripts/enums/JFCommonMessages.lua")

--------------------------------------------------------------------------------------------------
--										Startup function
--------------------------------------------------------------------------------------------------

function JFFrogOnOpen(posx, posy)
	local frog = JFFrog:initWithModel(nil)
	frog:spawnAtPosition(CGPoint(posx, posy))
end


--------------------------------------------------------------------------------------------------
--										JFFrog
--------------------------------------------------------------------------------------------------
waxClass{"JFFrog", JFActor, protocols = {"JFWalkingHandler"}}

nextIdentifier = 1.0


function initWithModel(self, model)

	self = self.super:initWithModel(model)
	if self then
		-- assign identifier
		self.identifier = nextIdentifier;
		nextIdentifier = nextIdentifier + 1.0
		
		-- setup state machine
		self:setupStateMachine()
		
		-- setup walk component
		self.walkComponent = JFWalkingComponent:initWithHandler(self)
	
		-- other initialization
	end
	return self
	
end


function classId(self)
    return JFUtility:convertClassToClassId(JFFrog:class())
end


function ident(self)
	return self.identifier
end


function walk(self)
	return self.walkComponent
end


function spawnAtPosition(self, tilePosition)

	-- get the required services
	local gos = JFServiceLocator:serviceOfType(JFGameObjectService:class()) 
	local map = JFServiceLocator:serviceOfType(JFMapService:class())
	
	-- set position
	self:setIsoPos(tilePosition)
	
	-- add to game object service and map
	gos:addGameObject(self)
	map:addActor(self)
	
	-- put the frog into the idle sitting state
	self:changeState(JFFrogStateIdle)
	
end


function changeState(self, stateClass)
	-- convenience function so this doens't have to be written every time
	self:stateMachine():setCurrentState_withMessage(stateClass:class(), nil)
end


function setupStateMachine(self)
	
	self:stateMachine():addState(JFFrogStateIdle:initWithFrog(self))
	
end



--
-- WALK DELEGATE
--
-- (float)walkSpeed;
function walkSpeed(self)
	return 1.0
end

-- (void)updateWalkingCostume:(enum JFActorDirection)aNewDirection;
function updateWalkingCostume(self, newDirection)
	self:updateWalkingCostume_force(newDirection, false)
end

-- (void)updateWalkingCostume:(enum JFActorDirection)aNewDirection force:(bool)aForce;
function updateWalkingCostume_force(self, newDirection, force)

    if self:direction() == newDirection and force == false then
    	return
    end
    
    local backWalk = "FROG_HOP_BACK"
    local frontWalk = "FROG_HOP"
    
    self:faceDirection(newDirection)
    
    if newDirection == JFActorDirection_West or newDirection == JFActorDirection_North then
    	self:wardrobe():playAnimation(backWalk)
    else
    	self:wardrobe():playAnimation(frontWalk)
    end
   	
end


--------------------------------------------------------------------------------------------------
--										JFFrogStateBase
--------------------------------------------------------------------------------------------------
waxClass{"JFFrogStateBase", JFStateBase}

function initWithFrog(self, froggie)
	self = self.super:init()
	if self then
		self.frog = froggie
	end
	return self
end


function getFrog(self)
	return self.frog
end


--------------------------------------------------------------------------------------------------
--										JFFrogStateIdle
--------------------------------------------------------------------------------------------------
waxClass{"JFFrogStateIdle", JFFrogStateBase}


function onEnter_withMessage(self, previousState, message)
	self.super:onEnter_withMessage(previousState, message)

	local frog = self:getFrog()
	
	-- set costume
	frog:wardrobe():playAnimation("FROG_IDLE")
	frog:updateSortingDepth()
	
	self:walkSomewhere()
	
end



function handleMessage(self, message)

	if message:name() == kMessage_WalkComplete then
		self:walkSomewhere()
	end

end



function walkSomewhere(self)

	local mapService = JFServiceLocator:serviceOfType(JFMapService:class())
	local tile = mapService:getRandomTileInRadius_withMaxRadius_withMinRadius(self:getFrog():isoPos(), 10, 10)
	local walk = self:getFrog():walk()	
	walk:walkToTile_onArrive(tile, nil)

end



